/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Java2OWL;

import TestPackage.TestAddress;
import TestPackage.TestPerson;
import TestPackage.TestScience;
import TestPackage.TestStudent;
import TestPackage.TestStudyProgramme;
import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.semanticweb.owlapi.expression.ParserException;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyStorageException;
import org.semanticweb.owlapi.util.AutoIRIMapper;

/**
 *
 * @author ohlbach
 * 
 * <p>Attention: If some of the Test-classes have been changed, you must do J2OSynchronizerTest first.
 * This injects the synchroniser code again. 
 */
public class J2OIndividualManagerTest {
    
    public J2OIndividualManagerTest() {
    }
    static J2OManager manager;
    static J2OOntologyManager omg;
    static J2OClassManager clm;
    static OWLOntology extOntology;
    static J2OIndividualManager img;
    static J2OSynchroniser sync = new J2OSynchroniser();
    static final TestAddress addrMuc = new TestAddress("München","Leopoldstraße",(short)28);
    static final TestAddress addrStu = new TestAddress("Stuttgart","Humboldstraße",(short)8);
    static final TestAddress addrBer = new TestAddress("Berlin","Unter den Linden",(short)88);
    static final TestAddress addrHam = new TestAddress("Hamburg","Am Damm",(short)185);
    static IndividualWrapper maddrMuc,maddrStu,maddrBer,maddrHam;
    
   
    static final TestScience sciMath = new TestScience("Mathematics");
    static final TestScience sciInf = new TestScience("Informatics");
    static IndividualWrapper msciMath,msciInf;
             
    static final TestStudyProgramme baMath = new TestStudyProgramme("Bachelor",sciMath);
    static final TestStudyProgramme maMath = new TestStudyProgramme("Master",sciMath);
    static IndividualWrapper mbaMath,mmaMath;
    
    static final TestStudyProgramme baInf = new TestStudyProgramme("Bachelor",sciInf);
    static final TestStudyProgramme maInf = new TestStudyProgramme("Master",sciInf);
    static IndividualWrapper mbaInf,mmaInf;
    
    static final TestPerson persMue = new TestPerson(true,"Müller",addrMuc,1.75f);
    static final TestPerson persMay = new TestPerson(true,"Mayer",addrMuc,1.85f);
    static IndividualWrapper mpersMue,mpersMay;
    
    static final TestStudent stuZi = new TestStudent(true,"Zimmermann",addrStu,1.65f);
    static final TestStudent stuHu = new TestStudent(true,"Humboldt",addrBer,1.7f);
    static final TestStudent stuSch = new TestStudent(true,"Schmidt",addrHam,1.8f);
    static IndividualWrapper mstuZi,mstuHu,mstuSch;
    
    static void compileClasses() {
        String dir = "build/test/classes/TestPackage/";
        String[] args = {
                dir+"TestPerson.class", 
                dir+"TestStudent.class", 
                dir+"TestAddress.class", 
                dir+"TestStudyProgramme.class", 
                dir+"TestScience.class", 
                        "bgo", "test/Java2OWL/BackgroundOntology.owl",
                        "exof", "test/Java2OWL/ExtensionOntology3.owl",
                        "exoi", "http://www.lmu.de/ExtensionOntology3.owl"};
        J2OCompiler.main(args);
    }
    
    @BeforeClass
    public static void setUpClass() throws Exception {
        compileClasses(); 

        manager = new J2OManager("TestManager",J2OManager.Status.RUN,true);
        Logging logger = manager.getLogger();
        //logger.setHandler(Logging.Destination.FILE, "/home/o/ohlbach/tmp/log.txt");
        AutoIRIMapper  iriMapper = new AutoIRIMapper( new File("test/Java2OWL"),true);
        omg = manager.getOntologyManager();
        omg.addIRIMapper(iriMapper);
        clm = manager.getClassManager();
        String name = "test/Java2OWL/ExtensionOntology3.owl";
        extOntology = omg.loadExtensionOntology(name);
        omg.setReasoner("HermiT", true, false, 10000);
 
        J2OCompiler comp = manager.getCompiler();
        comp.class2OWL(TestStudent.class,TestAddress.class,TestScience.class,TestStudyProgramme.class);
        comp.finishTranslation(null);
        
        img = manager.getIndvidualManager();
       
        
        stuZi.setProgramme(baMath);stuZi.setSemester((byte)1);
        stuHu.setProgramme(maMath);stuHu.setSemester((byte)3);
        stuSch.setProgramme(baInf);stuSch.setSemester((byte)5);
        
        stuZi.addFriend(stuHu);
        stuZi.addFriend(stuSch);
        
        clm.activateSynchronisation(TestAddress.class,TestPerson.class,TestStudent.class,TestScience.class,TestStudyProgramme.class);
  
        
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    /**
     * Test of object2OWL method, of class J2OIndividualManager.
     */
    @Test
    public void testObject2OWL() throws Exception {
        System.out.println("object2OWL");
        maddrMuc = img.object2OWL(addrMuc);
        System.out.println("addrMuc " +maddrMuc);
        assertEquals(addrMuc, maddrMuc.getFirstObject());
        assertEquals("München_Leopoldstraße_28", maddrMuc.getIndividual().getIRI().getFragment());
        
        maddrStu = img.object2OWL(addrStu);
        System.out.println("addrStu " +maddrStu);
        assertEquals(addrStu, maddrStu.getFirstObject());
        assertEquals("Stuttgart_Humboldstraße_8", maddrStu.getIndividual().getIRI().getFragment());
      
        maddrBer = img.object2OWL(addrBer);
        System.out.println("addrBer " +maddrBer);
        assertEquals(addrBer, maddrBer.getFirstObject());
        assertEquals("Berlin_Unter_den_Linden_88", maddrBer.getIndividual().getIRI().getFragment());
          
        maddrHam = img.object2OWL(addrHam);
        System.out.println("addrHam " +maddrHam);
        assertEquals(addrHam, maddrHam.getFirstObject());
        assertEquals("Hamburg_Am_Damm_185", maddrHam.getIndividual().getIRI().getFragment());
        
        msciMath = img.object2OWL(sciMath);
        System.out.println("sciMath " +msciMath);
        assertEquals(sciMath, msciMath.getFirstObject());
        assertEquals("Mathematics", msciMath.getIndividual().getIRI().getFragment());
        
        msciInf = img.object2OWL(sciInf);
        System.out.println("sciInf " +msciInf);
        assertEquals(sciInf, msciInf.getFirstObject());
        assertEquals("Informatics", msciInf.getIndividual().getIRI().getFragment());
        
        mbaMath = img.object2OWL(baMath);
        System.out.println("baMath " +mbaMath);
        assertEquals(baMath, mbaMath.getFirstObject());  
        assertEquals("Bachelor_Mathematics", mbaMath.getIndividual().getIRI().getFragment());
        
        mmaMath = img.object2OWL(maMath);
        System.out.println("maMath " +mmaMath);
        assertEquals(maMath, mmaMath.getFirstObject());
        assertEquals("Master_Mathematics", mmaMath.getIndividual().getIRI().getFragment());
        
        mmaInf = img.object2OWL(maInf);
        System.out.println("maInf " +mmaInf);
        assertEquals(maInf, mmaInf.getFirstObject());
        assertEquals("Master_Informatics", mmaInf.getIndividual().getIRI().getFragment());
        
        mbaInf = img.object2OWL(baInf);
        System.out.println("baInf " +mbaInf);
        assertEquals(baInf, mbaInf.getFirstObject());
        assertEquals("Bachelor_Informatics", mbaInf.getIndividual().getIRI().getFragment());
        
        mpersMue = img.object2OWL(persMue);
        System.out.println("persMue " +mpersMue);
        assertEquals(persMue, mpersMue.getFirstObject());
        assertEquals("Müller", mpersMue.getIndividual().getIRI().getFragment());
        
        mpersMay = img.object2OWL(persMay);
        System.out.println("persMay " +mpersMay);
        assertEquals(persMay, mpersMay.getFirstObject());
        assertEquals("Mayer", mpersMay.getIndividual().getIRI().getFragment());
        
        mstuZi = img.object2OWL(stuZi);
        System.out.println("stuZi " +mstuZi);
        assertEquals(stuZi, mstuZi.getFirstObject());
        assertEquals("Zimmermann", mstuZi.getIndividual().getIRI().getFragment());
        
        mstuHu = img.object2OWL(stuHu);
        System.out.println("stuHu " +mstuHu);
        assertEquals(stuHu, mstuHu.getFirstObject());
        assertEquals("Humboldt", mstuHu.getIndividual().getIRI().getFragment());
        
        mstuSch = img.object2OWL(stuSch);
        System.out.println("stuSch " +mstuSch);
        assertEquals(stuSch, mstuSch.getFirstObject());
        assertEquals("Schmidt", mstuSch.getIndividual().getIRI().getFragment());
        
        omg.saveExtensionOntology(); 
       
    }
    
    /**
     * Test of getIWRs method, of class J2OIndividualManager.
     */
    @Test
    public void testGetIMDs() {
        System.out.println("getIMDs");
        Object[] result; Set<IndividualWrapper> imds;
        result = img.getIWRs(stuZi);
        imds = (Set<IndividualWrapper>)result[0];
        assertEquals(1, imds.size());
        assertEquals(mstuZi, imds.iterator().next());
        
        result = img.getIWRs(stuZi,stuHu);
        imds = (Set<IndividualWrapper>)result[0];
        assertEquals(2, imds.size());
        assertEquals(true,imds.contains(mstuZi));
        assertEquals(true,imds.contains(mstuHu));
        
        Set<TestStudent> stdss = new HashSet<TestStudent>();
        result = img.getIWRs(stdss);
        imds = (Set<IndividualWrapper>)result[0];
        assertEquals(0, imds.size());
        stdss.add(stuZi); stdss.add(stuHu);
        result = img.getIWRs(stuZi,stuHu);
        imds = (Set<IndividualWrapper>)result[0];
        assertEquals(2, imds.size());
        assertEquals(true,imds.contains(mstuZi));
        assertEquals(true,imds.contains(mstuHu));
    }



    /**
     * Test of getIWR method, of class J2OIndividualManager.
     */
    @Test
    public void testGetIMD_Individual() {
        System.out.println("getIMD/Individual");
        IndividualWrapper imd = img.getIWR(mstuZi.getIndividual());
        assertEquals(mstuZi, imd);
        imd = img.getIWR(maddrBer.getIndividual());
        assertEquals(maddrBer, imd); 
    }
    
     /**
     * Test of getIWR method, of class J2OIndividualManager.
     */
    @Test
    public void testGetIMD_Object() {
        System.out.println("getIMD/Object");
        IndividualWrapper imd = img.getIWR(stuZi);
        assertEquals(mstuZi, imd);
        imd = img.getIWR(addrBer);
        assertEquals(maddrBer, imd); 
    }
    
    
  /**
     * Test of getInstances method, of class J2OIndividualManager.
     */
    @Test
    public void testGetInstances() throws Exception {
        System.out.println("getInstances");
        ArrayList<IndividualWrapper> imds = img.getInstances("Person");
        assertEquals(5, imds.size());
        assertEquals(true,imds.contains(mstuZi));
        
        imds = img.getInstances("TestPerson_hasFriends some LivingThing");
        assertEquals(1, imds.size());
        assertEquals(true,imds.contains(mstuZi));
        
    }
     
   
    
    /**
     * Test of changePropertyValue method, of class J2OIndividualManager.
     */
    @Test
    public void testChangePropertyValue() throws NoSuchMethodException, OWLOntologyStorageException, ParserException, J2OException {
        System.out.println("changePropertyValue");
       
        Object object = null;
        stuZi.setGender(false);
        assertEquals(false, stuZi.isMale());        
       
        omg.saveExtensionOntology();
        
        ArrayList<IndividualWrapper> imds = img.getInstances("isMale only {false}");
        assertEquals(1, imds.size());
        assertEquals(true,imds.contains(mstuZi));
        
        stuZi.addFriend(persMue);
        assertEquals(true, stuZi.hasFriends().contains(persMue));
        
        imds = img.getInstances("TestPerson_hasFriends some {Müller}");
        assertEquals(1, imds.size());
        assertEquals(true,imds.contains(mstuZi));
        
        imds = img.getInstances("TestPerson_hasFriends some {Humboldt}");
        assertEquals(1, imds.size());
        
        stuZi.removeFriend(stuHu);
        
        imds = img.getInstances("TestPerson_hasFriends some {Humboldt}");
        assertEquals(0, imds.size());
        
        addrBer.setCity("Frankfurt");
        assertEquals("Frankfurt", addrBer.getCity());
        
        imds = img.getInstances("livesInCity  only {\"Frankfurt\"}");
        assertEquals(1, imds.size());
        assertEquals(true,imds.contains(maddrBer));
    }

    
    
    /**
     * Test of individual2Java method, of class J2OIndividualManager.
     */
    @Test
    public void testIndividual2Java() throws ParserException {
        System.out.println("individual2Java");
        
        ArrayList<IndividualWrapper> addresses = img.getInstances("Address");
        ArrayList<IndividualWrapper> persons = img.getInstances("Person");
        System.out.println("PE " +persons.size());
        img.clear();
        for(IndividualWrapper addr : addresses) {
            TestAddress address = (TestAddress)((img.individual2Java(addr.getIndividual())).getFirstObject());
            assertNotNull(address);
            assertEquals(true,address.equals(addrBer) || address.equals(addrHam) || 
                    address.equals(addrMuc) || address.equals(addrStu));}

         for(IndividualWrapper pers : persons) {
            TestPerson person = (TestPerson)((img.individual2Java(pers.getIndividual())).getFirstObject());
            assertEquals(true,person.equals(persMay) || person.equals(persMue) || 
                    person.equals(stuHu) || person.equals(stuSch)|| person.equals(stuZi));}

        
        
    }

    

  
}
